@@ -1,20 +1,21 @@
-/*
- *    GDMXnestChooser - run X nest with a chooser using xdmcp
- *    (c)2001 Queen of England
+/* GDM - The GNOME Display Manager
+ * Copyright (c) 2001 Queen of England
  *    
- *    This program is free software; you can redistribute it and/or modify
- *    it under the terms of the GNU General Public License as published by
- *    the Free Software Foundation; either version 2 of the License, or
- *    (at your option) any later version.
+ * GDMXnestChooser - run X nest with a chooser using xdmcp
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *   
- *    This program is distributed in the hope that it will be useful,
- *    but WITHOUT ANY WARRANTY; without even the implied warranty of
- *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *   
- *    You should have received a copy of the GNU General Public License
- *    along with this program; if not, write to the Free Software
- *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
  */
 
@@ -383,7 +384,7 @@ get_auth_entry (int disp, char *cookie)
 	xa->address = malloc (strlen (hostname) + 1);
 	if (xa->address == NULL) {
 		free (xa);
-		return FALSE;
+		return NULL;
 	}
 
 	strcpy (xa->address, hostname);
@@ -402,7 +403,7 @@ get_auth_entry (int disp, char *cookie)
 		free (xa->name);
 		free (xa->address);
 		free (xa);
-		return FALSE;
+		return NULL;
 	}
 	memcpy (xa->data, cookie, 16);
 	xa->data_length = 16;
@@ -675,7 +676,7 @@ main (int argc, char *argv[])
 		_exit (1);
 	}
 
-	ve_waitpid_no_signal (xnest_pid, 0, 0);
+	ve_waitpid_no_signal (xnest_pid, NULL, 0);
 	xnest_pid = 0;
 
 	socket = g_strdup_printf ("/tmp/.X11-unix/X%d", display);
