@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This file Copyright (c) 2003 George Lebl
@@ -37,8 +37,8 @@
 extern gchar *GdmInfoMsgFile;
 extern gchar *GdmInfoMsgFont;
 extern gchar *GdmSoundProgram;
-extern gboolean GdmSoundOnLogin;
-extern gchar *GdmSoundOnLoginFile;
+extern gboolean GdmSoundOnLoginReady;
+extern gchar *GdmSoundOnLoginReadyFile;
 
 void
 gdm_common_show_info_msg (void)
@@ -111,11 +111,12 @@ gdm_common_message (const gchar *msg)
 	gdm_wm_no_login_focus_pop ();
 }
 
-gboolean
+gint
 gdm_common_query (const gchar *msg,
 		  gboolean markup,
 		  const char *posbutton,
-		  const char *negbutton)
+		  const char *negbutton,
+		  gboolean has_cancel)
 {
 	int ret;
 	GtkWidget *req;
@@ -132,31 +133,42 @@ gdm_common_query (const gchar *msg,
 				 msg,
 				 /* avoid warning */ "%s", "");
 
-	button = gtk_button_new_from_stock (negbutton);
-	gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_NO);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-	gtk_widget_show (button);
+	if (negbutton != NULL) {
+		button = gtk_button_new_from_stock (negbutton);
+		gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_NO);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+	}
 
-	button = gtk_button_new_from_stock (posbutton);
-	gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_YES);
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-	gtk_widget_show (button);
+	if (posbutton != NULL) {
+		button = gtk_button_new_from_stock (posbutton);
+		gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_YES);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+	}
 
-	gtk_dialog_set_default_response (GTK_DIALOG (req),
-					 GTK_RESPONSE_YES);
+	if (has_cancel == TRUE) {
+		button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+		gtk_dialog_add_action_widget (GTK_DIALOG (req), button, GTK_RESPONSE_CANCEL);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+	}
+
+	if (posbutton != NULL)
+		gtk_dialog_set_default_response (GTK_DIALOG (req), GTK_RESPONSE_YES);
+	else if (negbutton != NULL)
+		gtk_dialog_set_default_response (GTK_DIALOG (req), GTK_RESPONSE_NO);
+	else if (has_cancel)
+		gtk_dialog_set_default_response (GTK_DIALOG (req), GTK_RESPONSE_CANCEL);
 
 	gdm_wm_center_window (GTK_WINDOW (req));
 
 	gdm_wm_no_login_focus_push ();
 	ret = gtk_dialog_run (GTK_DIALOG (req));
 	gdm_wm_no_login_focus_pop ();
-
 	gtk_widget_destroy (req);
 
-	if (ret == GTK_RESPONSE_YES)
-		return TRUE;
-	else /* this includes window close */
-		return FALSE;
+	return ret;
 }
 
 void
@@ -229,14 +241,14 @@ gdm_common_int_same (VeConfig *config, int cur, const char *key)
 void
 gdm_common_login_sound (void)
 {
-	if ( ! GdmSoundOnLogin)
+	if ( ! GdmSoundOnLoginReady)
 		return;
 
 	if (ve_string_empty (g_getenv ("GDM_IS_LOCAL")) ||
 	    ve_string_empty (GdmSoundProgram) ||
-	    ve_string_empty (GdmSoundOnLoginFile) ||
+	    ve_string_empty (GdmSoundOnLoginReadyFile) ||
 	    access (GdmSoundProgram, F_OK) != 0 ||
-	    access (GdmSoundOnLoginFile, F_OK) != 0) {
+	    access (GdmSoundOnLoginReadyFile, F_OK) != 0) {
 		gdk_beep ();
 	} else {
 		/* login sound interruption */
@@ -348,3 +360,4 @@ gdm_setup_blinking_entry (GtkWidget *entry)
 
 	entries = g_slist_prepend (entries, eb);
 }
+
