@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This file Copyright (c) 2001 George Lebl
@@ -25,8 +25,10 @@
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#ifdef HAVE_LIBXINERAMA
+#ifdef HAVE_XFREE_XINERAMA
 #include <X11/extensions/Xinerama.h>
+#elif HAVE_SOLARIS_XINERAMA
+#include <X11/extensions/xinerama.h>
 #endif
 #include <pwd.h>
 #include <unistd.h>
@@ -63,6 +65,7 @@ static Atom XA_WM_PROTOCOLS = 0;
 static Atom XA_WM_STATE = 0;
 static Atom XA_WM_TAKE_FOCUS = 0;
 static Atom XA_COMPOUND_TEXT = 0;
+static Atom XA_NET_WM_STRUT = 0;
 
 static int trap_depth = 0;
 
@@ -70,6 +73,9 @@ GdkRectangle *gdm_wm_allscreens = NULL;
 int gdm_wm_screens = 0;
 GdkRectangle gdm_wm_screen = {0,0,0,0};
 
+static Window strut_owners[4] = {None, None, None, None};
+static guint save_struts[4] = {0, 0, 0, 0};
+
 void 
 gdm_wm_screen_init (int cur_screen_num)
 {
@@ -91,7 +97,7 @@ gdm_wm_screen_init (int cur_screen_num)
 	}
 
 	{
-#ifdef HAVE_LIBXINERAMA
+#ifdef HAVE_XFREE_XINERAMA
 	gboolean have_xinerama = FALSE;
 
 	gdk_flush ();
@@ -140,6 +146,67 @@ gdm_wm_screen_init (int cur_screen_num)
 
 		XFree (xscreens);
 	} else
+#elif HAVE_SOLARIS_XINERAMA
+	gboolean have_xinerama = FALSE;
+ 	/* This code from GDK, Copyright (C) 2002 Sun Microsystems */
+ 	int opcode;
+	int firstevent;
+	int firsterror;
+	int n_monitors = 0;
+	
+	gdk_flush ();
+	gdk_error_trap_push ();
+	have_xinerama = XQueryExtension (GDK_DISPLAY (),
+			"XINERAMA",
+			&opcode,
+			&firstevent,
+			&firsterror);
+	gdk_flush ();
+	if (gdk_error_trap_pop () != 0)
+		have_xinerama = FALSE;
+
+	if (have_xinerama) {
+		int i;
+		int result;
+		XRectangle monitors[MAXFRAMEBUFFERS];
+		unsigned char  hints[16];
+		
+		result = XineramaGetInfo (GDK_DISPLAY (), 0, monitors, hints, &n_monitors);
+		/* Yes I know it should be Success but the current implementation 
+		 * returns the num of monitor
+		 */
+
+		if (result <= 0) {
+			/* should NEVER EVER happen */
+			syslog (LOG_ERR, "Xinerama active, but <= 0 screens?");
+			gdm_wm_screen.x = 0;
+			gdm_wm_screen.y = 0;
+			gdm_wm_screen.width = gdk_screen_width ();
+			gdm_wm_screen.height = gdk_screen_height ();
+
+			gdm_wm_allscreens = g_new0 (GdkRectangle, 1);
+			gdm_wm_allscreens[0] = gdm_wm_screen;
+			gdm_wm_screens = 1;
+			return;
+		}
+
+		if (n_monitors <= cur_screen_num)
+			cur_screen_num = 0;
+
+		gdm_wm_allscreens = g_new0 (GdkRectangle, n_monitors);
+		gdm_wm_screens = n_monitors;
+
+		for (i = 0; i < n_monitors; i++) {
+			gdm_wm_allscreens[i].x = monitors[i].x;
+			gdm_wm_allscreens[i].y = monitors[i].y;
+			gdm_wm_allscreens[i].width = monitors[i].width;
+			gdm_wm_allscreens[i].height = monitors[i].height;
+
+			if (cur_screen_num == i)
+				gdm_wm_screen = gdm_wm_allscreens[i];
+		}
+
+	} else
 #endif
 	{
 		gdm_wm_screen.x = 0;
@@ -167,13 +234,13 @@ gdm_wm_set_screen (int cur_screen_num)
 void
 gdm_wm_center_window (GtkWindow *cw) 
 {
-	GtkRequisition req;
         gint x, y;
+        gint w, h;
 
-	gtk_widget_size_request (GTK_WIDGET (cw), &req);
+	gtk_window_get_size (cw, &w, &h); 
 
-	x = gdm_wm_screen.x + (gdm_wm_screen.width - req.width)/2;
-	y = gdm_wm_screen.y + (gdm_wm_screen.height - req.height)/2;	
+	x = gdm_wm_screen.x + (gdm_wm_screen.width - w)/2;
+	y = gdm_wm_screen.y + (gdm_wm_screen.height - h)/2;	
 
 	if (x < gdm_wm_screen.x)
 		x = gdm_wm_screen.x;
@@ -317,6 +384,34 @@ get_typed_property_data (Display *xdisplay,
   return data;
 }
 
+/* 
+ * Update the gdm_wm_screen 'effective' screen area when a window reserves struts.
+ * This only works if struts don't "collide", i.e. there is a max of one strut setter
+ * per edge.  Of course this should be the case at gdm time...
+ */
+static void
+gdm_wm_update_struts (Display *xdisplay, Window xwindow)
+{
+	gint     size = 0;
+	guint32 *struts = get_typed_property_data (xdisplay, xwindow, XA_NET_WM_STRUT, 
+						   XA_CARDINAL, &size, 32);
+	if (size == 16)
+	{
+		gint i;
+		for (i = 0; i < 4; ++i) 
+		{
+			/* strut owners are the only windows whose 'zero' struts are reflected */
+			if (struts[i] != 0 || (strut_owners[i] == xwindow))
+			{
+			/* if any window re-specifies a strut, it becomes the new owner */
+				strut_owners[i] = xwindow;
+				save_struts[i] = struts[i];
+			}
+		}
+	}
+	g_free (struts);
+}
+
 /* stolen from gwmh */
 static gboolean
 wm_protocol_check_support (Window xwin,
@@ -442,6 +537,71 @@ gdm_wm_focus_window (Window window)
 	wm_focus_window = window;
 }
 
+static void
+constrain_window (GdmWindow *gw)
+{
+/* constrain window to lie within screen geometry, with struts reserved */
+	int x, y, screen_x = 0, screen_y = 0;
+	Window root;
+	unsigned int width, height, border, depth;
+	unsigned int screen_width = gdk_screen_width (), screen_height = gdk_screen_height ();
+
+	/* exclude any strut areas not owned by this window */
+	if (strut_owners[0] != gw->win) 
+	{
+		screen_x = save_struts[0];
+		screen_width -= save_struts[0];
+	}
+	if (strut_owners[2] != gw->win) 
+	{
+		screen_y = save_struts[2];
+		screen_height -= save_struts[2];
+	}
+	if (strut_owners[1] != gw->win) 
+		screen_width -= save_struts[1];
+	if (strut_owners[3] != gw->win) 
+		screen_height -= save_struts[3];
+	
+	if (gw->deco == None) 
+	    return;
+
+	trap_push ();
+
+	XGetGeometry (wm_disp, gw->deco,
+		      &root, &x, &y, &width, &height, &border, &depth);
+
+	if (width > screen_width)
+	    width = screen_width;
+	if (height > screen_height)
+	    height = screen_height;
+	
+	if (x < screen_x)
+	    x = screen_x;
+	if (y < screen_y)
+	    y = screen_y;
+	if ((x - screen_x + width) > screen_width)
+	    x = screen_width - width;
+	if ((y - screen_y + height) > screen_height)
+	    y = screen_height - height;
+
+	XMoveResizeWindow (wm_disp, gw->deco, x, y, width, height);
+
+	trap_pop ();
+}
+
+static void
+constrain_all_windows (void)
+{
+	GList *winlist = windows;
+
+	while (winlist)
+	{
+		GdmWindow *gw = winlist->data;
+		constrain_window (gw);
+		winlist = winlist->next;
+	}
+}
+
 static void
 center_x_window (GdmWindow *gw, Window w, Window hintwin)
 {
@@ -537,6 +697,7 @@ static gboolean
 has_deco (Window win)
 {
 	static Atom hints_atom = None;
+	unsigned char *foo;
 	MotifWmHints *hints;
 	Atom type;
 	gint format;
@@ -555,7 +716,8 @@ has_deco (Window win)
 			    hints_atom, 0,
 			    sizeof (MotifWmHints) / sizeof (long),
 			    False, AnyPropertyType, &type, &format, &nitems,
-			    &bytes_after, (guchar **)&hints);
+			    &bytes_after, &foo);
+	hints = (MotifWmHints *)foo;
 
 	if (type != None &&
 	    hints != NULL &&
@@ -584,6 +746,11 @@ add_deco (GdmWindow *w, gboolean is_mapped)
 
 	trap_push ();
 
+	XGetWindowAttributes (wm_disp, w->win, &attribs);
+	XSelectInput (wm_disp, w->win,
+		      attribs.your_event_mask |
+		      PropertyChangeMask);
+
 	if ( ! has_deco (w->win)) {
 		trap_pop ();
 		return;
@@ -619,6 +786,7 @@ add_deco (GdmWindow *w, gboolean is_mapped)
 	XSelectInput (wm_disp, w->deco,
 		      attribs.your_event_mask |
 		      EnterWindowMask |
+		      PropertyChangeMask |
 		      SubstructureNotifyMask |
 		      SubstructureRedirectMask);
 
@@ -1012,6 +1180,14 @@ event_process (XEvent *ev)
 		if (gw != NULL)
 			gdm_wm_focus_window (gw->win);
 		break;
+	case PropertyNotify:
+		if (ev->xproperty.atom == XA_NET_WM_STRUT)
+		{
+			gdm_wm_update_struts (ev->xproperty.display, 
+					      ev->xproperty.window);
+			constrain_all_windows ();
+		}
+		break;
 	default:
 		break;
 	}
@@ -1027,14 +1203,14 @@ event_prepare (GSource *source,
 	       gint     *timeout)
 {
 	*timeout = -1;
-	return XPending (wm_disp);
+	return XPending (wm_disp) > 0;
 }
 
 static gboolean  
 event_check (GSource *source)
 {
 	if (event_poll_fd.revents & G_IO_IN) {
-		return XPending (wm_disp);
+		return XPending (wm_disp) > 0;
 	} else {
 		return FALSE;
 	}
@@ -1043,7 +1219,7 @@ event_check (GSource *source)
 static void
 process_events (void)
 {
-	while (XPending (wm_disp)) {
+	while (XPending (wm_disp) > 0) {
 		XEvent ev;
 		XNextEvent (wm_disp, &ev);
 		event_process (&ev);
@@ -1102,6 +1278,10 @@ gdm_wm_init (Window login_window)
 
 	wm_login_window = login_window;
 
+	if (wm_disp != NULL) {
+		return;
+	}
+
 	wm_disp = XOpenDisplay (gdk_get_display ());
 	if (wm_disp == NULL) {
 		/* EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEK! */
@@ -1116,6 +1296,7 @@ gdm_wm_init (Window login_window)
 	XA_WM_TAKE_FOCUS = XInternAtom (wm_disp, "WM_TAKE_FOCUS", False);
 
 	XA_COMPOUND_TEXT = XInternAtom (wm_disp, "COMPOUND_TEXT", False);
+	XA_NET_WM_STRUT = XInternAtom (wm_disp, "_NET_WM_STRUT", False);
 
 	wm_root = DefaultRootWindow (wm_disp);
 
