@@ -1,3 +1,21 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <libintl.h>
@@ -7,6 +25,13 @@
 #include <unistd.h>
 #include <signal.h>
 
+#if HAVE_PAM
+#include <security/pam_appl.h>
+#define PW_ENTRY_SIZE PAM_MAX_RESP_SIZE
+#else
+#define PW_ENTRY_SIZE GDM_MAX_PASS
+#endif
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomecanvas/libgnomecanvas.h>
@@ -33,6 +58,7 @@
 #include "greeter_system.h"
 
 gboolean DOING_GDM_DEVELOPMENT = FALSE;
+static char *greeter_DefaultWelcome_key = GDM_KEY_DEFAULT_WELCOME;
 static char *greeter_Welcome_key = GDM_KEY_WELCOME;
 
 GtkWidget *window;
@@ -40,7 +66,9 @@ GtkWidget *canvas;
 
 gboolean GDM_IS_LOCAL = FALSE;
 char *GdmGraphicalTheme = NULL;
+char *GdmGraphicalThemes = NULL;
 char *GdmGraphicalThemeDir = NULL;
+gboolean GdmGraphicalThemeRand = FALSE;
 int GdmXineramaScreen = 0;
 gboolean GdmShowGnomeFailsafeSession = FALSE;
 gboolean GdmShowXtermFailsafeSession = FALSE;
@@ -69,20 +97,28 @@ gchar *GdmTimedLogin;
 gchar *GdmGtkRC;
 gchar *GdmGtkTheme;
 gint GdmTimedLoginDelay;
-gchar *GdmExclude;
 int GdmMinimalUID;
+gchar *GdmInclude = NULL;
+gchar *GdmExclude = NULL;
+gboolean GdmIncludeAll;
 gboolean GdmAllowRoot;
 gboolean GdmAllowRemoteRoot;
+static gboolean GdmDefaultWelcome;
 gchar *GdmWelcome;
 gchar *GdmServAuthDir;
 gchar *GdmInfoMsgFile;
 gchar *GdmInfoMsgFont;
 gint GdmFlexiReapDelayMinutes;
-gboolean GdmSoundOnLogin;
-gchar *GdmSoundOnLoginFile;
+gboolean GdmSoundOnLoginReady;
+gboolean GdmSoundOnLoginSuccess;
+gboolean GdmSoundOnLoginFailure;
+gchar *GdmSoundOnLoginReadyFile;
+gchar *GdmSoundOnLoginSuccessFile;
+gchar *GdmSoundOnLoginFailureFile;
 gchar *GdmSoundProgram;
 
 gboolean GdmUseCirclesInEntry = FALSE;
+gboolean GdmUseInvisibleInEntry = FALSE;
 
 static gboolean used_defaults = FALSE;
 gint greeter_current_delay = 0;
@@ -95,6 +131,38 @@ extern gboolean require_quarter;
 
 gboolean greeter_probably_login_prompt = FALSE;
 
+/* If in random theme mode then grab a random theme from those selected */
+static char *
+get_random_theme ()
+{
+    char **vec;
+	char *themes_list;
+	char *theme;
+    int size;
+	int i;
+
+    VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
+	themes_list = ve_config_get_string (config, GDM_KEY_GRAPHICAL_THEMES);
+
+    if (ve_string_empty (themes_list))
+        return NULL;
+
+    vec = g_strsplit (themes_list, GDM_DELIMITER_THEMES, -1);
+    if (vec == NULL)
+        return NULL;
+
+	/* Get Number of elements in vector */
+    for (size = 0; vec[size] != NULL; size++) {}
+
+	/* Get Random Theme from list */
+	srand ( time(NULL) );
+	i = rand() % size;
+	theme = g_strdup (vec[i]);
+    g_strfreev (vec);
+
+    return theme;
+}
+
 static void 
 greeter_parse_config (void)
 {
@@ -109,19 +177,29 @@ greeter_parse_config (void)
 
     if (ve_string_empty (g_getenv ("GDM_IS_LOCAL"))) {
 	    greeter_Welcome_key = GDM_KEY_REMOTEWELCOME;
+	    greeter_DefaultWelcome_key = GDM_KEY_DEFAULT_REMOTEWELCOME;
     } else {
 	    greeter_Welcome_key = GDM_KEY_WELCOME;
+	    greeter_DefaultWelcome_key = GDM_KEY_DEFAULT_WELCOME;
     }
 
     config = ve_config_get (GDM_CONFIG_FILE);
 
-    GdmGraphicalTheme = ve_config_get_string (config, GDM_KEY_GRAPHICAL_THEME);
-    if (GdmGraphicalTheme == NULL ||
-	GdmGraphicalTheme[0] == '\0')
-      {
-	g_free (GdmGraphicalTheme);
-	GdmGraphicalTheme = g_strdup ("circles");
-      }
+	GdmGraphicalThemeRand = ve_config_get_bool (config,
+	                                            GDM_KEY_GRAPHICAL_THEME_RAND);
+
+	if (GdmGraphicalThemeRand)
+		GdmGraphicalTheme = get_random_theme(); 
+	else
+		GdmGraphicalTheme = ve_config_get_string (config,
+		                                          GDM_KEY_GRAPHICAL_THEME);
+
+	if (GdmGraphicalTheme == NULL || GdmGraphicalTheme[0] == '\0')
+	{
+		g_free (GdmGraphicalTheme);
+		GdmGraphicalTheme = g_strdup ("circles");
+	}
+
 
     GdmGraphicalThemeDir = ve_config_get_string (config, GDM_KEY_GRAPHICAL_THEME_DIR);
     if (GdmGraphicalThemeDir == NULL ||
@@ -132,6 +210,7 @@ greeter_parse_config (void)
       }
     GdmXineramaScreen = ve_config_get_int (config, GDM_KEY_XINERAMASCREEN);
     GdmUseCirclesInEntry = ve_config_get_bool (config, GDM_KEY_ENTRY_CIRCLES);
+    GdmUseInvisibleInEntry = ve_config_get_bool (config, GDM_KEY_ENTRY_INVISIBLE);
 
     GdmShowXtermFailsafeSession = ve_config_get_bool (config, GDM_KEY_SHOW_XTERM_FAILSAFE);
     GdmShowGnomeFailsafeSession = ve_config_get_bool (config, GDM_KEY_SHOW_GNOME_FAILSAFE);
@@ -158,16 +237,21 @@ greeter_parse_config (void)
     GdmConfiguratorFound = gdm_working_command_exists (GdmConfigurator);
 
     GdmWelcome = ve_config_get_translated_string (config, greeter_Welcome_key);
-    /* A hack! */
-    if (strcmp (ve_sure_string (GdmWelcome), "Welcome") == 0) {
+    GdmDefaultWelcome = ve_config_get_bool (config, greeter_DefaultWelcome_key);
+
+    /* Replace default welcome message with one specified in the config file, if
+     * use default is set to no */
+    if (GdmDefaultWelcome && strcmp (greeter_Welcome_key, GDM_KEY_WELCOME) == 0) {
 	    g_free (GdmWelcome);
-	    GdmWelcome = g_strdup (_("Welcome"));
-    } else if (strcmp (ve_sure_string (GdmWelcome), "Welcome to %n") == 0) {
+	    GdmWelcome = g_strdup (_(GDM_DEFAULT_WELCOME_MSG));
+    } else if (GdmDefaultWelcome && strcmp (greeter_Welcome_key, GDM_KEY_REMOTEWELCOME) == 0) {
 	    g_free (GdmWelcome);
-	    GdmWelcome = g_strdup (_("Welcome to %n"));
+	    GdmWelcome = g_strdup (_(GDM_DEFAULT_REMOTEWELCOME_MSG));
     }
 
     GdmTimedLoginEnable = ve_config_get_bool (config, GDM_KEY_TIMED_LOGIN_ENABLE);
+    GdmIncludeAll = ve_config_get_bool (config, GDM_KEY_INCLUDEALL);
+    GdmInclude = ve_config_get_string (config, GDM_KEY_INCLUDE);
     GdmExclude = ve_config_get_string (config, GDM_KEY_EXCLUDE);
     GdmMinimalUID = ve_config_get_int (config, GDM_KEY_MINIMALUID);
     GdmAllowRoot = ve_config_get_bool (config, GDM_KEY_ALLOWROOT);
@@ -209,8 +293,18 @@ greeter_parse_config (void)
     GdmDefaultFace = ve_config_get_string (config, GDM_KEY_FACE);
 
     GdmSoundProgram = ve_config_get_string (config, GDM_KEY_SOUND_PROGRAM);
-    GdmSoundOnLogin = ve_config_get_bool (config, GDM_KEY_SOUND_ON_LOGIN);
-    GdmSoundOnLoginFile = ve_config_get_string (config, GDM_KEY_SOUND_ON_LOGIN_FILE);
+    GdmSoundOnLoginReady = ve_config_get_bool (config,
+	                       GDM_KEY_SOUND_ON_LOGIN_READY);
+    GdmSoundOnLoginSuccess = ve_config_get_bool (config,
+	                         GDM_KEY_SOUND_ON_LOGIN_SUCCESS);
+    GdmSoundOnLoginFailure = ve_config_get_bool (config,
+	                         GDM_KEY_SOUND_ON_LOGIN_FAILURE);
+    GdmSoundOnLoginReadyFile = ve_config_get_string (config,
+	                           GDM_KEY_SOUND_ON_LOGIN_READY_FILE);
+    GdmSoundOnLoginSuccessFile = ve_config_get_string (config,
+	                      GDM_KEY_SOUND_ON_LOGIN_SUCCESS_FILE);
+    GdmSoundOnLoginFailureFile = ve_config_get_string (config,
+	                      GDM_KEY_SOUND_ON_LOGIN_FAILURE_FILE);
 
     if (GdmXineramaScreen < 0)
       GdmXineramaScreen = 0;
@@ -265,10 +359,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 	
 	greeter_item_pam_set_user (buf);
-	if (ve_string_empty (buf))
-	  greeter_item_ulist_enable ();
-	else
-	  greeter_item_ulist_disable ();
+	greeter_item_ulist_enable ();
 	printf ("%c\n", STX);
 	fflush (stdout);
 	break;
@@ -284,7 +375,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	} else {
 		greeter_probably_login_prompt = FALSE;
 	}
-	greeter_item_pam_prompt (tmp, 128, TRUE);
+	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, TRUE);
 	g_free (tmp);
 	break;
 
@@ -293,7 +384,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 
 	tmp = ve_locale_to_utf8 (buf);
-	greeter_item_pam_prompt (tmp, 128, FALSE);
+	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, FALSE);
 	g_free (tmp);
 	break;
 
@@ -305,7 +396,6 @@ greeter_ctrl_handler (GIOChannel *source,
 	g_free (tmp);
 	printf ("%c\n", STX);
 	fflush (stdout);
-
 	break;
 
     case GDM_ERRBOX:
@@ -355,18 +445,26 @@ greeter_ctrl_handler (GIOChannel *source,
 	tmp = ve_locale_to_utf8 (buf);
 	session = greeter_session_lookup (tmp);
 	g_free (tmp);
-	tmp = ve_locale_from_utf8 (session);
-	printf ("%c%s\n", STX, tmp);
+
+	if (greeter_save_session() == GTK_RESPONSE_CANCEL) {
+	     printf ("%c%s\n", STX, GDM_RESPONSE_CANCEL);
+	} else {
+	    tmp = ve_locale_from_utf8 (session);
+	    printf ("%c%s\n", STX, tmp);
+	    g_free (tmp);
+	}
 	fflush (stdout);
 	g_free (session);
-	g_free (tmp);
 	break;
 
     case GDM_LANG:
         g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 	buf[len-1] = '\0';
 	language = greeter_language_get_language (buf);
-	printf ("%c%s\n", STX, language);
+	if (greeter_language_get_save_language() == GTK_RESPONSE_CANCEL)
+	    printf ("%c%s\n", STX, GDM_RESPONSE_CANCEL);
+	else
+	    printf ("%c%s\n", STX, language);
 	fflush (stdout);
 	g_free (language);
 	break;
@@ -374,7 +472,7 @@ greeter_ctrl_handler (GIOChannel *source,
     case GDM_SSESS:
         g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
-	if (greeter_save_session ())
+	if (greeter_save_session() == GTK_RESPONSE_YES)
 	  printf ("%cY\n", STX);
 	else
 	  printf ("%c\n", STX);
@@ -385,7 +483,7 @@ greeter_ctrl_handler (GIOChannel *source,
     case GDM_SLANG:
         g_io_channel_read_chars (source, buf, PIPE_SIZE-1, &len, NULL); /* Empty */
 
-	if (greeter_language_get_save_language ())
+	if (greeter_language_get_save_language() == GTK_RESPONSE_YES)
 	    printf ("%cY\n", STX);
 	else
 	    printf ("%c\n", STX);
@@ -577,6 +675,30 @@ key_press_event (GtkWidget *widget, GdkEventKey *key, gpointer data)
   return FALSE;
 }
 
+static void
+greeter_action_ok (GreeterItemInfo *info,
+                       gpointer         user_data)
+{
+   GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+   if (entry_info && entry_info->item &&
+       GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
+       GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
+     {
+       GtkWidget *entry;
+       entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+       greeter_item_pam_login (GTK_ENTRY (entry), entry_info);
+     }
+}
+
+static void
+greeter_action_cancel (GreeterItemInfo *info,
+                       gpointer         user_data)
+{
+   greeter_item_ulist_disable ();
+   printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_CANCEL);
+   fflush (stdout);
+}
+
 static void
 greeter_setup_items (void)
 {
@@ -588,6 +710,12 @@ greeter_setup_items (void)
 
   greeter_item_capslock_setup (window);
   greeter_item_timed_setup ();
+  greeter_item_register_action_callback ("ok_button",
+					 greeter_action_ok,
+					 window);
+  greeter_item_register_action_callback ("cancel_button",
+					 greeter_action_cancel,
+					 window);
   greeter_item_register_action_callback ("language_button",
 					 greeter_action_language,
 					 window);
@@ -638,7 +766,7 @@ verify_gdm_version (void)
 				  _("The greeter version (%s) does not match the daemon "
 				    "version.\n"
 				    "You have probably just upgraded gdm.\n"
-				    "Please restart the gdm daemon or reboot the computer."),
+				    "Please restart the gdm daemon or the computer."),
 				  VERSION);
     
       gtk_widget_show_all (dialog);
@@ -670,7 +798,7 @@ verify_gdm_version (void)
 				  _("The greeter version (%s) does not match the daemon "
 				    "version.\n"
 				    "You have probably just upgraded gdm.\n"
-				    "Please restart the gdm daemon or reboot the computer."),
+				    "Please restart the gdm daemon or the computer."),
 				  VERSION);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Reboot"),
@@ -715,7 +843,7 @@ verify_gdm_version (void)
 				  _("The greeter version (%s) does not match the daemon "
 				    "version (%s).\n"
 				    "You have probably just upgraded gdm.\n"
-				    "Please restart the gdm daemon or reboot the computer."),
+				    "Please restart the gdm daemon or the computer."),
 				  VERSION, gdm_version);
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Restart"),
@@ -775,63 +903,30 @@ greeter_reread_config (int sig, gpointer data)
 	/* Also we may not need to check ALL those keys but just a few */
 	if (strcmp (theme, GdmGraphicalTheme) != 0 ||
 	    strcmp (theme_dir, GdmGraphicalThemeDir) != 0 ||
-	     ! gdm_common_string_same (config,
-			    GdmGtkRC,
-			    GDM_KEY_GTKRC) ||
-	     ! gdm_common_string_same (config,
-			    GdmGtkTheme,
-			    GDM_KEY_GTK_THEME) ||
-	     ! gdm_common_int_same (config,
-			 GdmXineramaScreen,
-			 GDM_KEY_XINERAMASCREEN) ||
-	     ! gdm_common_bool_same (config,
-			  GdmUseCirclesInEntry,
-			  GDM_KEY_ENTRY_CIRCLES) ||
-	     ! gdm_common_bool_same (config,
-			  GdmShowXtermFailsafeSession,
-			  GDM_KEY_SHOW_XTERM_FAILSAFE) ||
-	     ! gdm_common_bool_same (config,
-			  GdmShowGnomeFailsafeSession,
-			  GDM_KEY_SHOW_GNOME_FAILSAFE) ||
-	     ! gdm_common_string_same (config,
-			    GdmSessionDir,
-			    GDM_KEY_SESSDIR) ||
-	     ! gdm_common_string_same (config,
-			    GdmLocaleFile,
-			    GDM_KEY_LOCFILE) ||
-	     ! gdm_common_bool_same (config,
-			  GdmSystemMenu,
-			  GDM_KEY_SYSMENU) ||
-	     ! gdm_common_string_same (config,
-			    GdmHalt,
-			    GDM_KEY_HALT) ||
-	     ! gdm_common_string_same (config,
-			    GdmReboot,
-			    GDM_KEY_REBOOT) ||
-	     ! gdm_common_string_same (config,
-			    GdmSuspend,
-			    GDM_KEY_SUSPEND) ||
-	     ! gdm_common_string_same (config,
-			    GdmConfigurator,
-			    GDM_KEY_CONFIGURATOR) ||
-	     ! gdm_common_string_same (config,
-			    GdmInfoMsgFile,
-			    GDM_KEY_INFO_MSG_FILE) ||
-	     ! gdm_common_string_same (config,
-			    GdmInfoMsgFont,
-			    GDM_KEY_INFO_MSG_FONT) ||
-	     ! gdm_common_bool_same (config,
-			  GdmConfigAvailable,
-			  GDM_KEY_CONFIG_AVAILABLE) ||
-	     ! gdm_common_bool_same (config,
-			  GdmChooserButton,
-			  GDM_KEY_CHOOSER_BUTTON) ||
-	     ! gdm_common_bool_same (config,
-			  GdmTimedLoginEnable,
-			  GDM_KEY_TIMED_LOGIN_ENABLE) ||
-	     ! gdm_common_int_same (config,
-			 GdmTimedLoginDelay,
-			 GDM_KEY_TIMED_LOGIN_DELAY)) {
+	     ! gdm_common_string_same (config, GdmGtkRC, GDM_KEY_GTKRC) ||
+	     ! gdm_common_string_same (config, GdmGtkTheme, GDM_KEY_GTK_THEME) ||
+	     ! gdm_common_int_same (config, GdmXineramaScreen, GDM_KEY_XINERAMASCREEN) ||
+	     ! gdm_common_bool_same (config, GdmUseCirclesInEntry, GDM_KEY_ENTRY_CIRCLES) ||
+	     ! gdm_common_bool_same (config, GdmUseInvisibleInEntry, GDM_KEY_ENTRY_INVISIBLE) ||
+	     ! gdm_common_bool_same (config, GdmShowXtermFailsafeSession, GDM_KEY_SHOW_XTERM_FAILSAFE) ||
+	     ! gdm_common_bool_same (config, GdmShowGnomeFailsafeSession, GDM_KEY_SHOW_GNOME_FAILSAFE) ||
+	     ! gdm_common_bool_same (config, GdmIncludeAll, GDM_KEY_INCLUDEALL) ||
+	     ! gdm_common_string_same (config, GdmInclude, GDM_KEY_INCLUDE) ||
+	     ! gdm_common_string_same (config, GdmExclude, GDM_KEY_EXCLUDE) ||
+	     ! gdm_common_string_same (config, GdmSessionDir, GDM_KEY_SESSDIR) ||
+	     ! gdm_common_string_same (config, GdmLocaleFile, GDM_KEY_LOCFILE) ||
+	     ! gdm_common_bool_same (config, GdmSystemMenu, GDM_KEY_SYSMENU) ||
+	     ! gdm_common_string_same (config, GdmHalt, GDM_KEY_HALT) ||
+	     ! gdm_common_string_same (config, GdmReboot, GDM_KEY_REBOOT) ||
+	     ! gdm_common_string_same (config, GdmSuspend, GDM_KEY_SUSPEND) ||
+	     ! gdm_common_string_same (config, GdmConfigurator, GDM_KEY_CONFIGURATOR) ||
+	     ! gdm_common_string_same (config, GdmInfoMsgFile, GDM_KEY_INFO_MSG_FILE) ||
+	     ! gdm_common_string_same (config, GdmInfoMsgFont, GDM_KEY_INFO_MSG_FONT) ||
+	     ! gdm_common_bool_same (config, GdmConfigAvailable, GDM_KEY_CONFIG_AVAILABLE) ||
+	     ! gdm_common_bool_same (config, GdmChooserButton, GDM_KEY_CHOOSER_BUTTON) ||
+	     ! gdm_common_bool_same (config, GdmTimedLoginEnable, GDM_KEY_TIMED_LOGIN_ENABLE) ||
+	     ! gdm_common_int_same (config, GdmTimedLoginDelay, GDM_KEY_TIMED_LOGIN_DELAY)) {
+
 		/* Set busy cursor */
 		gdm_common_setup_cursor (GDK_WATCH);
 
@@ -841,8 +936,18 @@ greeter_reread_config (int sig, gpointer data)
 	}
 
 	GdmSoundProgram = ve_config_get_string (config, GDM_KEY_SOUND_PROGRAM);
-	GdmSoundOnLogin = ve_config_get_bool (config, GDM_KEY_SOUND_ON_LOGIN);
-	GdmSoundOnLoginFile = ve_config_get_string (config, GDM_KEY_SOUND_ON_LOGIN_FILE);
+	GdmSoundOnLoginReady = ve_config_get_bool (config,
+	                  GDM_KEY_SOUND_ON_LOGIN_READY);
+	GdmSoundOnLoginSuccess = ve_config_get_bool (config,
+	                  GDM_KEY_SOUND_ON_LOGIN_SUCCESS);
+	GdmSoundOnLoginFailure = ve_config_get_bool (config,
+	                  GDM_KEY_SOUND_ON_LOGIN_FAILURE);
+	GdmSoundOnLoginReadyFile = ve_config_get_string (config,
+	                             GDM_KEY_SOUND_ON_LOGIN_READY_FILE);
+	GdmSoundOnLoginSuccessFile = ve_config_get_string (config,
+	                             GDM_KEY_SOUND_ON_LOGIN_SUCCESS_FILE);
+	GdmSoundOnLoginFailureFile = ve_config_get_string (config,
+	                             GDM_KEY_SOUND_ON_LOGIN_FAILURE_FILE);
 
 	if ( ! gdm_common_bool_same (config,
 				     GdmUse24Clock,
@@ -852,27 +957,51 @@ greeter_reread_config (int sig, gpointer data)
 		greeter_item_clock_update ();
 	}
 
-	str = ve_config_get_translated_string (config, greeter_Welcome_key);
-	/* A hack */
-	if (strcmp (ve_sure_string (str), "Welcome") == 0) {
-		g_free (str);
-		str = g_strdup (_("Welcome"));
-	} else if (strcmp (ve_sure_string (str), "Welcome to %n") == 0) {
-		g_free (str);
-		str = g_strdup (_("Welcome to %n"));
-	}
-	if (strcmp (ve_sure_string (str), ve_sure_string (GdmWelcome)) != 0) {
-		g_free (GdmWelcome);
-		GdmWelcome = str;
-		if (welcome_string_info != NULL) {
-			g_free (welcome_string_info->data.text.orig_text);
-			welcome_string_info->data.text.orig_text = g_strdup (str);
-			greeter_item_update_text (welcome_string_info);
+        if (! gdm_common_bool_same (config, GdmDefaultWelcome, greeter_DefaultWelcome_key) ||
+	   (!GdmDefaultWelcome && 
+	    ! gdm_common_string_same (config, GdmWelcome, greeter_Welcome_key))) {
+
+		GdmWelcome = ve_config_get_string (config, greeter_Welcome_key);
+		GdmDefaultWelcome = ve_config_get_bool (config, greeter_DefaultWelcome_key);
+
+		if (GdmDefaultWelcome) {
+			if (strcmp (greeter_Welcome_key, GDM_KEY_WELCOME) == 0) {
+				g_free (GdmWelcome);
+				GdmWelcome = g_strdup (_(GDM_DEFAULT_WELCOME_MSG));
+			} else if (strcmp (greeter_Welcome_key, GDM_KEY_REMOTEWELCOME) == 0) {
+				g_free (GdmWelcome);
+				GdmWelcome = g_strdup (_(GDM_DEFAULT_REMOTEWELCOME_MSG));
+			}
+
+			if (welcome_string_info != NULL) {
+				g_free (welcome_string_info->data.text.orig_text);
+				welcome_string_info->data.text.orig_text = g_strdup (GdmWelcome);
+				greeter_item_update_text (welcome_string_info);
+			}
+
+		} else {
+			str = ve_config_get_translated_string (config, greeter_Welcome_key);
+
+			if (strcmp (ve_sure_string (str), ve_sure_string (GdmWelcome)) != 0 && 
+			    welcome_string_info != NULL) {
+					g_free (welcome_string_info->data.text.orig_text);
+					welcome_string_info->data.text.orig_text = g_strdup (GdmWelcome);
+					greeter_item_update_text (welcome_string_info);
+			
+			} else {
+				g_free (str);
+			}
 		}
-	} else {
-		g_free (str);
+
+		/* Set busy cursor */
+		gdm_common_setup_cursor (GDK_WATCH);
+
+		gdm_wm_save_wm_order ();
+
+		_exit (DISPLAY_RESTARTGREETER);
 	}
 
+
 	g_free (theme);
 	g_free (theme_dir);
 
@@ -1024,6 +1153,50 @@ gdm_kill_thingies (void)
 	return;
 }
 
+static gboolean
+gdm_event (GSignalInvocationHint *ihint,
+           guint                n_param_values,
+           const GValue        *param_values,
+           gpointer             data)
+{
+        GdkEvent *event;
+
+        /* HAAAAAAAAAAAAAAAAACK */
+        /* Since the user has not logged in yet and may have left/right
+         * mouse buttons switched, we just translate every right mouse click
+         * to a left mouse click */
+        if (n_param_values != 2 ||
+            !G_VALUE_HOLDS (&param_values[1], GDK_TYPE_EVENT))
+          return FALSE;
+
+        event = g_value_get_boxed (&param_values[1]);
+        if ((event->type == GDK_BUTTON_PRESS ||
+             event->type == GDK_2BUTTON_PRESS ||
+             event->type == GDK_3BUTTON_PRESS ||
+             event->type == GDK_BUTTON_RELEASE)
+            && event->button.button == 3)
+                event->button.button = 1;
+
+        /* Support Ctrl-U for blanking the username/password entry */
+        if (event->type == GDK_KEY_PRESS &&
+            (event->key.state & GDK_CONTROL_MASK) &&
+            (event->key.keyval == GDK_u ||
+             event->key.keyval == GDK_U)) {
+
+		GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+		if (entry_info && entry_info->item &&
+		    GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
+		    GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
+		{
+			GtkWidget *entry;
+			entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+			gtk_entry_set_text (GTK_ENTRY (entry), "");
+		}
+	}
+
+        return TRUE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1036,6 +1209,7 @@ main (int argc, char *argv[])
   char *theme_file;
   char *theme_dir;
   const char *gdm_gtk_theme;
+  guint sid;
   int r;
 
   if (g_getenv ("DOING_GDM_DEVELOPMENT") != NULL)
@@ -1066,7 +1240,7 @@ main (int argc, char *argv[])
   if ( ! ve_string_empty (gdm_gtk_theme)) {
 	  gdm_set_theme (gdm_gtk_theme);
   }
-  
+
   gdm_wm_screen_init (GdmXineramaScreen);
   
   r = verify_gdm_version ();
@@ -1225,8 +1399,7 @@ main (int argc, char *argv[])
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
 				  FALSE /* markup */,
-				  _("The theme for the graphical greeter "
-				    "is corrupt"),
+				  _("The greeter theme is corrupt"),
 				  "%s",
 				  _("The theme does not contain "
 				    "definition for the username/password "
@@ -1264,8 +1437,8 @@ main (int argc, char *argv[])
 				  FALSE /* markup */,
 				  _("There was an error loading the "
 				    "theme, and the default theme "
-				    "also could not have been loaded, "
-				    "I will attempt to start the "
+				    "could not be loaded. "
+				    "Attempting to start the "
 				    "standard greeter"),
 				  /* avoid warning */ "%s", "");
     
@@ -1277,7 +1450,7 @@ main (int argc, char *argv[])
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
 
-      execl (EXPANDED_BINDIR "/gdmlogin", EXPANDED_BINDIR "/gdmlogin", NULL);
+      execl (EXPANDED_LIBEXECDIR "/gdmlogin", EXPANDED_LIBEXECDIR "/gdmlogin", NULL);
       execlp ("gdmlogin", "gdmlogin", NULL);
 
       dialog = ve_hig_dialog_new (NULL /* parent */,
@@ -1285,11 +1458,10 @@ main (int argc, char *argv[])
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
 				  FALSE /* markup */,
-				  _("I could not start the standard "
-				    "greeter.  This display will abort "
-				    "and you may have to login another "
-				    "way and fix the installation of "
-				    "gdm"),
+				  _("The GTK+ greeter could not be started.  "
+				    "This display will abort and you may "
+				    "have to login another way and fix the "
+				    "installation of gdm"),
 				  /* avoid warning */ "%s", "");
     
       gtk_widget_show_all (dialog);
@@ -1367,7 +1539,7 @@ main (int argc, char *argv[])
 				  _("Configuration is not correct"),
 				  "%s",
 				  _("The configuration file contains an invalid command "
-				    "line for the login dialog, and thus I ran the "
+				    "line for the login dialog, so running the "
 				    "default command.  Please fix your configuration."));
 
       gtk_widget_show_all (dialog);
@@ -1417,8 +1589,8 @@ main (int argc, char *argv[])
       /* but don't reap Xnest flexis */
       ve_string_empty (g_getenv ("GDM_PARENT_DISPLAY")))
     {
-      guint sid = g_signal_lookup ("activate",
-				   GTK_TYPE_MENU_ITEM);
+      sid = g_signal_lookup ("activate",
+			     GTK_TYPE_MENU_ITEM);
       g_signal_add_emission_hook (sid,
 				  0 /* detail */,
 				  delay_reaping,
@@ -1445,6 +1617,14 @@ main (int argc, char *argv[])
       g_timeout_add (60*1000, reap_flexiserver, NULL);
     }
 
+  sid = g_signal_lookup ("event",
+                               GTK_TYPE_WIDGET);
+  g_signal_add_emission_hook (sid,
+                              0 /* detail */,
+                              gdm_event,
+                              NULL /* data */,
+                              NULL /* destroy_notify */);
+
   gdm_wm_restore_wm_order ();
 
   gdm_common_show_info_msg ();
